'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _default11 = _interopDefault(require('ramda/src/last'));
var _default10 = _interopDefault(require('ramda/src/reduce'));
var _default9 = _interopDefault(require('ramda/src/__'));
var _default8 = _interopDefault(require('ramda/src/propIs'));
var _default7 = _interopDefault(require('ramda/src/filter'));
var _default6 = _interopDefault(require('ramda/src/forEach'));
var _default5 = _interopDefault(require('ramda/src/keys'));
var _default4 = _interopDefault(require('ramda/src/compose'));
var _default3 = _interopDefault(require('ramda/src/concat'));
var _default2 = _interopDefault(require('ramda/src/apply'));
var _default = _interopDefault(require('ramda/src/curry'));

var activeChanges = []; /* global sinon */

var isActiveChangeForObject = function isActiveChangeForObject(object) {
  return function (activeChange) {
    return activeChange.object === object;
  };
};

function getActiveChangesForObject(object) {
  var activeChangesForObject = activeChanges.filter(isActiveChangeForObject(object))[0];
  if (!activeChangesForObject) {
    activeChangesForObject = {
      object: object,
      activeStubs: {},
      activeReplacements: {}
    };
    activeChanges.push(activeChangesForObject);
  }
  return activeChangesForObject;
}

var createStubOrSpyForObjectKey = _default(function (stubbingFunction, object, args) {
  restoreKey(object, args[0]);
  var activeStubs = getActiveChangesForObject(object).activeStubs;
  activeStubs[args[0]] = _default2(stubbingFunction, _default3([object], args));
});

function replaceObjectKey(object, key, replacement) {
  restoreKey(object, key);
  var activeReplacements = getActiveChangesForObject(object).activeReplacements;
  activeReplacements[key] = object[key];
  object[key] = replacement;
}

function restoreKey(object, key) {
  var activeChangesForObject = getActiveChangesForObject(object);
  var activeStubs = activeChangesForObject.activeStubs;
  var activeReplacements = activeChangesForObject.activeReplacements;

  if (activeStubs.hasOwnProperty(key)) {
    activeStubs[key].restore();
    delete activeStubs[key];
  }
  if (activeReplacements.hasOwnProperty(key)) {
    object[key] = activeReplacements[key];
    delete activeReplacements[key];
  }
}

var applyToEachKeyInObject = function applyToEachKeyInObject(object, appliedFunction) {
  return _default4(_default6(appliedFunction), _default5)(object);
};

function restoreActiveChangesForObject(activeChangesForObject) {
  applyToEachKeyInObject(activeChangesForObject.activeStubs, function (key) {
    activeChangesForObject.activeStubs[key].restore();
  });
  applyToEachKeyInObject(activeChangesForObject.activeReplacements, function (key) {
    activeChangesForObject.object[key] = activeChangesForObject.activeReplacements[key];
  });
}

var applyToEachFunctionKeyInObject = function applyToEachFunctionKeyInObject(appliedFunction, object) {
  _default4(_default6(appliedFunction), _default7(_default8(Function, _default9, object)))(Object.getOwnPropertyNames(object));
};

var applyToEachFunctionKeyInPrototypeChain = function applyToEachFunctionKeyInPrototypeChain(appliedFunction, object, prototypeLevels) {
  if (object && prototypeLevels >= 0) {
    applyToEachFunctionKeyInObject(appliedFunction, object);
    applyToEachFunctionKeyInPrototypeChain(appliedFunction, Object.getPrototypeOf(object), prototypeLevels - 1);
  }
};

function getArrayFromArrayLikeObject(args) {
  return Array.prototype.slice.call(args);
}

// we cannot directly define this as the result of reduce because then, each call would use the SAME
// empty array reference as reduce initializer
var parseStringFunctionArrayToArguments = function parseStringFunctionArrayToArguments(argsArray) {
  return _default10(function (arrayOfArguments, argument) {
    if (typeof argument !== 'function') {
      arrayOfArguments.push([argument]);
    } else {
      _default11(arrayOfArguments).push(argument);
    }
    return arrayOfArguments;
  }, [])(argsArray);
};

function restore() {
  activeChanges.forEach(restoreActiveChangesForObject);
  activeChanges.length = 0;
}

function onObject(target) {
  var getTargetStubber = function getTargetStubber(stubbingFunction) {
    return function stubOrSpy() {
      var args = getArrayFromArrayLikeObject(arguments);

      if (args.length === 0) {
        applyToEachFunctionKeyInObject(stubOrSpy, target);
      } else if (args.length === 1 && typeof args[0] === 'number') {
        applyToEachFunctionKeyInPrototypeChain(stubOrSpy, target, args[0]);
      } else {
        _default4(_default6(createStubOrSpyForObjectKey(stubbingFunction, target)), parseStringFunctionArrayToArguments)(args);
      }
      return this;
    };
  };

  function replace(key, replacement) {
    replaceObjectKey(target, key, replacement);
    return this;
  }

  return {
    stub: getTargetStubber(sinon.stub),
    spy: getTargetStubber(sinon.spy),
    replace: replace
  };
}

exports.restore = restore;
exports.onObject = onObject;
